@@ -46,9 +46,9 @@ static unsigned char** saferkey=NULL; /* For safer64 and 128 */
 static int finished_threads=0;
 
 /* For CBC mode */
-static char **previous_ciphertext;
-static char **previous_plaintext;
-static char **previous_cipher;
+static word32 **previous_ciphertext;
+static word32 **previous_plaintext;
+static word32 **previous_cipher;
 
 /* For CFB mode */
 static char **sd_register;
@@ -130,10 +130,6 @@ int init_mcrypt(const int algorithm, void * key,const int lenofkey)
 char *keyword1, *keyword2, *keyword3; /* For 3DES and safer*/
 char *skey=NULL;
 
-cur_thread++;
-
-if (cur_thread < MAX_THREADS && finished_threads==0) {
-
   saferkey=realloc(saferkey, (cur_thread+1)*sizeof(char*));
   kn1=realloc(kn1, (cur_thread+1)*sizeof(char*));
   kn2=realloc(kn2, (cur_thread+1)*sizeof(char*));
@@ -142,15 +138,7 @@ if (cur_thread < MAX_THREADS && finished_threads==0) {
   ckey=realloc(ckey, (cur_thread+1) * sizeof(struct cast_key));  
   keyword_given=realloc(keyword_given, (cur_thread+1)*sizeof(char*));
   algorithm_given=realloc(algorithm_given, (cur_thread+1)*sizeof(int));
-}
-else {
-  cur_thread=0;
-  finished_threads=1;
-  while (keyword_given[cur_thread]!=NULL && cur_thread <= MAX_THREADS) {
-   cur_thread++;
-  }  
-  if (cur_thread >= MAX_THREADS) return -1;
-}
+
 
   algorithm_given[cur_thread]=algorithm;
   keyword_given[cur_thread]=calloc(1, get_key_size(algorithm));
@@ -199,7 +187,9 @@ else {
       
    case TripleDES:
       desinit(0);
-      keyword1=malloc(8); keyword2=malloc(8); keyword3=malloc(8);
+      keyword1=calloc(1,8*sizeof(char)); 
+      keyword2=calloc(1,8*sizeof(char)); 
+      keyword3=calloc(1,8*sizeof(char));
       kn1[cur_thread]=malloc(8*16*sizeof(char));
       kn2[cur_thread]=malloc(8*16*sizeof(char));
       kn3[cur_thread]=malloc(8*16*sizeof(char));
@@ -220,8 +210,12 @@ else {
       break;
        
    case BLOWFISH:
-      if (lenofkey<2) return (-1); 
-      blf_key(&c[cur_thread],keyword_given[cur_thread],lenofkey);
+      if (lenofkey<2) {
+          blf_key(&c[cur_thread],keyword_given[cur_thread],2);
+      }
+      else {
+          blf_key(&c[cur_thread],keyword_given[cur_thread],lenofkey);
+      }
       return cur_thread;
       break;
 
@@ -242,6 +236,10 @@ else {
 }
 
 
+
+
+
+
 /* plaintext should be in block's size */
 int mcrypt(int thread, void *plaintext)
 {
@@ -395,17 +393,42 @@ keyword_given[td]=NULL;
 return 0;
 }
 
-
 /* ECB MODE */
 
+int init_mcrypt_ecb(const int algorithm, void * key,const int lenofkey)
+{
+
+cur_thread++;
+
+  if (cur_thread < MAX_THREADS && finished_threads==0) {
+
+  }
+  else {
+     cur_thread=0;
+     finished_threads=1;
+  
+     while (keyword_given[cur_thread]!=NULL && cur_thread <= MAX_THREADS) {
+       cur_thread++;
+     }  
+  
+     if (cur_thread >= MAX_THREADS) return -1;
+  }
+
+return init_mcrypt(algorithm, key, lenofkey);
+
+}
+
+
+
+
 int mcrypt_ecb(int td, void *plaintext, int len)
 {
-int j;
+int j,blocksize=get_block_size(algorithm_given[td]);
 char *plain=plaintext;
 
- for (j=0; j<len/get_block_size(algorithm_given[td]); j++) {
+ for (j=0; j<len/blocksize; j++) {
 
-  mcrypt(td, &plain[j*get_block_size(algorithm_given[td])]);
+  mcrypt(td, &plain[j*blocksize]);
 
  }
   return 0;
@@ -415,12 +438,12 @@ char *plain=plaintext;
 
 int mdecrypt_ecb(int td, void *ciphertext, int len)
 {
-int j;
+int j, blocksize=get_block_size(algorithm_given[td]);
 char *cipher=ciphertext;
 
- for (j=0; j<len/get_block_size(algorithm_given[td]); j++) {
+ for (j=0; j<len/blocksize; j++) {
 
-    mdecrypt(td, &cipher[j*get_block_size(algorithm_given[td])]);
+    mdecrypt(td, &cipher[j*blocksize]);
 
  }
 
@@ -435,72 +458,28 @@ char *cipher=ciphertext;
 int end_mcrypt_cbc(int td)
 {
 
-free(keyword_given[td]); 
-keyword_given[td]=NULL;
 free(previous_ciphertext[td]);
 free(previous_cipher[td]);
 free(previous_plaintext[td]);
 
-   switch(algorithm_given[td]) {
-  
-   case DES:
-	desdone();
-	free(kn1[td]);
-      break;
+return end_mcrypt(td);
 
-   case TripleDES:
-	desdone();
-	free(kn1[td]);
-	free(kn2[td]);
-	free(kn3[td]);
-      break;
-      
-   case BLOWFISH:
-   case TEAN:   
-	break;
-
-   case GOST:
-   case ThreeWAY:
-	break;
-   case CAST128:
-        break;
-   case SAFER64:
-   case SAFER128:
-	free(saferkey[td]);   
-	break;
-      break;
-
-   default:
-      return 0;
-  }
-
-return 0;
 }
 
 
 int init_mcrypt_cbc(const int algorithm, void * key,const int lenofkey)
 {
 
-char *keyword1, *keyword2, *keyword3; /* For 3DES and safer*/
-char *skey=NULL;
-
 cur_thread++;
 
 if (cur_thread < MAX_THREADS && finished_threads==0) {
 
-  saferkey=realloc(saferkey, (cur_thread+1)*sizeof(char*));
-  kn1=realloc(kn1, (cur_thread+1)*sizeof(char*));
-  kn2=realloc(kn2, (cur_thread+1)*sizeof(char*));
-  kn3=realloc(kn3, (cur_thread+1)*sizeof(char*));
 /* for cbc */
-  previous_ciphertext=realloc(previous_ciphertext, (cur_thread+1) * sizeof(char*));
-  previous_cipher=realloc(previous_cipher, (cur_thread+1) * sizeof(char*));
-  previous_plaintext=realloc(previous_plaintext, (cur_thread+1) * sizeof(char*));
+  previous_ciphertext=realloc(previous_ciphertext, (cur_thread+1) * sizeof(word32*));
+  previous_cipher=realloc(previous_cipher, (cur_thread+1) * sizeof(word32*));
+  previous_plaintext=realloc(previous_plaintext, (cur_thread+1) * sizeof(word32*));
 /* end cbc */
-  c=realloc(c, (cur_thread+1) * sizeof(blf_ctx));
-  ckey=realloc(ckey, (cur_thread+1) * sizeof(struct cast_key));  
-  keyword_given=realloc(keyword_given, (cur_thread+1)*sizeof(char*));
-  algorithm_given=realloc(algorithm_given, (cur_thread+1)*sizeof(int));
+
 }
 else {
   cur_thread=0;
@@ -511,113 +490,28 @@ else {
   if (cur_thread >= MAX_THREADS) return -1;
 }
 
-  algorithm_given[cur_thread]=algorithm;
-  keyword_given[cur_thread]=calloc(1, get_key_size(algorithm));
-  memmove(keyword_given[cur_thread], key, lenofkey);
-  skey=keyword_given[cur_thread];
-
 /* For cbc */
-  previous_ciphertext[cur_thread]=calloc(1,get_block_size(algorithm_given[cur_thread]));
-  previous_cipher[cur_thread]=malloc(get_block_size(algorithm_given[cur_thread]));
-  previous_plaintext[cur_thread]=calloc(1,get_block_size(algorithm_given[cur_thread]));
+  previous_ciphertext[cur_thread]=calloc(1,get_block_size(algorithm));
+  previous_cipher[cur_thread]=malloc(get_block_size(algorithm));
+  previous_plaintext[cur_thread]=calloc(1,get_block_size(algorithm));
 
 /* End cbc */
 
-
-   switch(algorithm) {
-  
-   case DES:
-      desinit(0);
-      kn1[cur_thread]=malloc(8*16*sizeof(char));
-      
-      des_setkey( (void*) kn1[cur_thread], keyword_given[cur_thread]);
-      return cur_thread;
-      break;
-
-   case SAFER64:
-      keyword1=malloc(8);
-      memmove(keyword1, keyword_given[cur_thread], 8);
-      saferkey[cur_thread]=malloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
-
-      Safer_Init_Module();
-      Safer_Expand_Userkey( (unsigned char*) keyword1,(unsigned char*) keyword1, SAFER_SK64_DEFAULT_NOF_ROUNDS, 1, saferkey[cur_thread]);
-      /* 8 rounds and secure key schedule */
-      free(keyword1);
-
-   case CAST128:
-      cast_setkey( &ckey[cur_thread], (void*)keyword_given[cur_thread], lenofkey);
-      return cur_thread;
-      break; 
-
-   case SAFER128:
-      Safer_Init_Module();
-      keyword1=malloc(8); keyword2=malloc(8);
-      saferkey[cur_thread]=malloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
-      
-      memmove(keyword1, skey, 8);
-      memmove(keyword2, &skey[8], 8);
-      Safer_Expand_Userkey( (unsigned char*)keyword1, (unsigned char*)keyword2, SAFER_SK128_DEFAULT_NOF_ROUNDS, 1, saferkey[cur_thread]);
-      /* 8 rounds and secure key schedule */
-      free(keyword1); free(keyword2);
-
-      return cur_thread;
-      break;
-      
-   case TripleDES:
-      desinit(0);
-      keyword1=malloc(8); keyword2=malloc(8); keyword3=malloc(8);
-      kn1[cur_thread]=malloc(8*16*sizeof(char));
-      kn2[cur_thread]=malloc(8*16*sizeof(char));
-      kn3[cur_thread]=malloc(8*16*sizeof(char));
-      
-      /* If it returns 1 or -1 the key is too small */
-
-      if (BreakToThree(keyword_given[cur_thread], lenofkey, keyword1, keyword2, keyword3)!=0) {
-        free(keyword1); free(keyword2); free(keyword3);
-        end_mcrypt(cur_thread);
-        return (-1);
-      }
-      des_setkey( (void*)kn1[cur_thread], keyword1);
-      des_setkey( (void*)kn2[cur_thread], keyword2);
-      des_setkey( (void*)kn3[cur_thread], keyword3);
-      free(keyword1); free(keyword2); free(keyword3);
-
-      return cur_thread;
-      break;
-       
-   case BLOWFISH:
-      if (lenofkey<2) return (-1); 
-      blf_key(&c[cur_thread],keyword_given[cur_thread],lenofkey);
-      return cur_thread;
-      break;
-
-   case GOST:
-      kboxinit();
-      return cur_thread;
-      break;
-
-   case ThreeWAY:
-   case TEAN:
-      return cur_thread;
-      break;
-
-   default:
-      return -1;
-  }
+return init_mcrypt(algorithm, key, lenofkey);
 
 }
 
 int mcrypt_cbc(int td, void *plaintext, int len)
 {
-  char* fplain=plaintext;
-  char* plain;
-  int i,j;
+  word32* fplain=plaintext;
+  word32* plain;
+  int i,j, blocksize=get_block_size(algorithm_given[td]);
 
- for (j=0; j<len/get_block_size(algorithm_given[td]); j++) {
+ for (j=0; j<len/blocksize; j++) {
 
-  plain=&fplain[j*get_block_size(algorithm_given[td])];
+  plain=&fplain[j*blocksize/4];
   
-  for (i=0; i<get_block_size(algorithm_given[td]); i++) {
+  for (i=0 ; i< blocksize/4 ; i++) {
     plain[i] ^= previous_ciphertext[td][i];
   }
 
@@ -625,7 +519,7 @@ int mcrypt_cbc(int td, void *plaintext, int len)
 
 
   /* Copy the ciphertext to prev_ciphertext */
-  memmove(previous_ciphertext[td], plain, get_block_size(algorithm_given[td]));
+  memmove(previous_ciphertext[td], plain, blocksize);
  }
  
   return 0;
@@ -635,22 +529,22 @@ int mcrypt_cbc(int td, void *plaintext, int len)
 
 int mdecrypt_cbc(int td, void *ciphertext, int len)
 {
-  char* cipher;
-  char *fcipher=ciphertext;
-  int i,j;
+  word32* cipher;
+  word32 *fcipher=ciphertext;
+  int i,j, blocksize=get_block_size(algorithm_given[td]);
 
- for (j=0; j<len/get_block_size(algorithm_given[td]); j++) {
+ for (j=0; j<len/blocksize; j++) {
 
-    cipher=&fcipher[j*get_block_size(algorithm_given[td])];
-    memmove(previous_cipher[td], cipher, get_block_size(algorithm_given[td]));  
+    cipher=&fcipher[j*blocksize/4];
+    memmove(previous_cipher[td], cipher, blocksize);  
   
     mdecrypt(td, cipher);
-    for (i=0; i<get_block_size(algorithm_given[td]); i++) {
+    for (i=0; i< blocksize/4; i++) {
       cipher[i] ^= previous_plaintext[td][i];
     }
 
     /* Copy the plaintext to prev_plaintext */
-    memmove(previous_plaintext[td], previous_cipher[td], get_block_size(algorithm_given[td]));  
+    memmove(previous_plaintext[td], previous_cipher[td], blocksize);  
 
  }
 
@@ -664,74 +558,29 @@ int mdecrypt_cbc(int td, void *ciphertext, int len)
 int end_mcrypt_cfb(int td)
 {
 
-free(keyword_given[td]); 
-keyword_given[td]=NULL;
 free(s_register[td]);
 free(enc_s_register[td]);
 free(enc_sd_register[td]);
 free(sd_register[td]);
 
-   switch(algorithm_given[td]) {
-  
-   case DES:
-	desdone();
-	free(kn1[td]);
-      break;
+return end_mcrypt(td);
 
-   case TripleDES:
-	desdone();
-	free(kn1[td]);
-	free(kn2[td]);
-	free(kn3[td]);
-      break;
-      
-   case BLOWFISH:
-   case TEAN:   
-	break;
-
-   case GOST:
-   case ThreeWAY:
-	break;
-   case CAST128:
-        break;
-   case SAFER64:
-   case SAFER128:
-	free(saferkey[td]);   
-	break;
-      break;
-
-   default:
-      return 0;
-  }
-
-return 0;
 }
 
 
 int init_mcrypt_cfb(const int algorithm, void * key,const int lenofkey, void *IV)
 {
 
-char *keyword1, *keyword2, *keyword3; /* For 3DES and safer*/
-char *skey=NULL;
-
 cur_thread++;
 
 if (cur_thread < MAX_THREADS && finished_threads==0) {
 
-  saferkey=realloc(saferkey, (cur_thread+1)*sizeof(char*));
-  kn1=realloc(kn1, (cur_thread+1)*sizeof(char*));
-  kn2=realloc(kn2, (cur_thread+1)*sizeof(char*));
-  kn3=realloc(kn3, (cur_thread+1)*sizeof(char*));
 /* for cfb */
   s_register=realloc(s_register, (cur_thread+1) * sizeof(char*));
   enc_s_register=realloc(enc_s_register, (cur_thread+1) * sizeof(char*));  
   enc_sd_register=realloc(enc_sd_register, (cur_thread+1) * sizeof(char*));  
   sd_register=realloc(sd_register, (cur_thread+1) * sizeof(char*));
 /* end cfb */
-  c=realloc(c, (cur_thread+1) * sizeof(blf_ctx));
-  ckey=realloc(ckey, (cur_thread+1) * sizeof(struct cast_key));  
-  keyword_given=realloc(keyword_given, (cur_thread+1)*sizeof(char*));
-  algorithm_given=realloc(algorithm_given, (cur_thread+1)*sizeof(int));
 }
 else {
   cur_thread=0;
@@ -742,123 +591,38 @@ else {
   if (cur_thread >= MAX_THREADS) return -1;
 }
 
-  algorithm_given[cur_thread]=algorithm;
-  keyword_given[cur_thread]=calloc(1, get_key_size(algorithm));
-  memmove(keyword_given[cur_thread], key, lenofkey);
-  skey=keyword_given[cur_thread];
-
 /* For cfb */
-  s_register[cur_thread]=malloc( get_block_size(algorithm_given[cur_thread]));
-  sd_register[cur_thread]=malloc( get_block_size(algorithm_given[cur_thread]));
-  enc_s_register[cur_thread]=malloc( get_block_size(algorithm_given[cur_thread]));
-  enc_sd_register[cur_thread]=malloc( get_block_size(algorithm_given[cur_thread]));
-  memmove(sd_register[cur_thread], IV, get_block_size(algorithm_given[cur_thread]));
-  memmove(s_register[cur_thread], IV, get_block_size(algorithm_given[cur_thread]));  
+  s_register[cur_thread]=malloc( get_block_size(algorithm));
+  sd_register[cur_thread]=malloc( get_block_size(algorithm));
+  enc_s_register[cur_thread]=malloc( get_block_size(algorithm));
+  enc_sd_register[cur_thread]=malloc( get_block_size(algorithm));
+  memmove(sd_register[cur_thread], IV, get_block_size(algorithm));
+  memmove(s_register[cur_thread], IV, get_block_size(algorithm));  
 
 /* End cfb */
 
-
-   switch(algorithm) {
-  
-   case DES:
-      desinit(0);
-      kn1[cur_thread]=malloc(8*16*sizeof(char));
-      
-      des_setkey( (void*) kn1[cur_thread], keyword_given[cur_thread]);
-      return cur_thread;
-      break;
-
-   case SAFER64:
-      keyword1=malloc(8);
-      memmove(keyword1, keyword_given[cur_thread], 8);
-      saferkey[cur_thread]=malloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
-
-      Safer_Init_Module();
-      Safer_Expand_Userkey( (unsigned char*) keyword1,(unsigned char*) keyword1, SAFER_SK64_DEFAULT_NOF_ROUNDS, 1, saferkey[cur_thread]);
-      /* 8 rounds and secure key schedule */
-      free(keyword1);
-
-   case CAST128:
-      cast_setkey( &ckey[cur_thread], (void*)keyword_given[cur_thread], lenofkey);
-      return cur_thread;
-      break; 
-
-   case SAFER128:
-      Safer_Init_Module();
-      keyword1=malloc(8); keyword2=malloc(8);
-      saferkey[cur_thread]=malloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
-      
-      memmove(keyword1, skey, 8);
-      memmove(keyword2, &skey[8], 8);
-      Safer_Expand_Userkey( (unsigned char*)keyword1, (unsigned char*)keyword2, SAFER_SK128_DEFAULT_NOF_ROUNDS, 1, saferkey[cur_thread]);
-      /* 8 rounds and secure key schedule */
-      free(keyword1); free(keyword2);
-
-      return cur_thread;
-      break;
-      
-   case TripleDES:
-      desinit(0);
-      keyword1=malloc(8); keyword2=malloc(8); keyword3=malloc(8);
-      kn1[cur_thread]=malloc(8*16*sizeof(char));
-      kn2[cur_thread]=malloc(8*16*sizeof(char));
-      kn3[cur_thread]=malloc(8*16*sizeof(char));
-      
-      /* If it returns 1 or -1 the key is too small */
-
-      if (BreakToThree(keyword_given[cur_thread], lenofkey, keyword1, keyword2, keyword3)!=0) {
-        free(keyword1); free(keyword2); free(keyword3);
-        end_mcrypt(cur_thread);
-        return (-1);
-      }
-      des_setkey( (void*)kn1[cur_thread], keyword1);
-      des_setkey( (void*)kn2[cur_thread], keyword2);
-      des_setkey( (void*)kn3[cur_thread], keyword3);
-      free(keyword1); free(keyword2); free(keyword3);
-
-      return cur_thread;
-      break;
-       
-   case BLOWFISH:
-      if (lenofkey<2) return (-1); 
-      blf_key(&c[cur_thread],keyword_given[cur_thread],lenofkey);
-      return cur_thread;
-      break;
-
-   case GOST:
-      kboxinit();
-      return cur_thread;
-      break;
-
-   case ThreeWAY:
-   case TEAN:
-      return cur_thread;
-      break;
-
-   default:
-      return -1;
-  }
+return init_mcrypt(algorithm, key, lenofkey);
 
 }
 
 int mcrypt_cfb(int td, void *plaintext,int len) /* plaintext is 1 byte (8bit cfb) */
 {
   char* plain=plaintext;
-  int i,j;
+  int i,j,blocksize=get_block_size(algorithm_given[td]);;
 
  for (j=0;j<len;j++) {
   
-  memmove(enc_s_register[td], s_register[td], get_block_size(algorithm_given[td]));
+  memmove(enc_s_register[td], s_register[td], blocksize);
   
   mcrypt(td, enc_s_register[td]);
 
   plain[j] ^= enc_s_register[td][0];
   
 /* Shift the register */
-  for (i=0 ; i< (get_block_size(algorithm_given[td])-1) ; i++) 
+  for (i=0 ; i< (blocksize-1) ; i++) 
     s_register[td][i]=s_register[td][i+1];
   
-  s_register[td][get_block_size(algorithm_given[td])-1]=plain[j];
+  s_register[td][blocksize-1]=plain[j];
 
 
  }
@@ -871,19 +635,19 @@ int mcrypt_cfb(int td, void *plaintext,int len) /* plaintext is 1 byte (8bit cfb
 int mdecrypt_cfb(int td, void *plaintext,int len) /* plaintext is 1 byte (8bit cfb) */
 {
   char* plain=plaintext;
-  int i,j;
+  int i,j,blocksize=get_block_size(algorithm_given[td]);;
   
  for (j=0;j<len;j++) {
  
-  memmove(enc_sd_register[td], sd_register[td], get_block_size(algorithm_given[td]));
+  memmove(enc_sd_register[td], sd_register[td], blocksize);
 
   mcrypt(td, enc_sd_register[td]);
 
 /* Shift the register */
-  for (i=0 ; i<(get_block_size(algorithm_given[td])-1) ; i++) 
+  for (i=0 ; i<(blocksize-1) ; i++) 
     sd_register[td][i]=sd_register[td][i+1];
   
-  sd_register[td][get_block_size(algorithm_given[td])-1]=plain[j];
+  sd_register[td][blocksize-1]=plain[j];
 
   plain[j] ^= enc_sd_register[td][0];
 
