@@ -1,5 +1,4 @@
 #define end_mcrypt_ecb end_mcrypt
-#define init_mcrypt_ecb init_mcrypt
 
 int get_block_size(int algorithm);
 int get_key_size(int algorithm); /* In bytes */
@@ -10,6 +9,7 @@ int mdecrypt(int thread, void *plaintext);
 int end_mcrypt(int td);
 
 /* ECB */
+int init_mcrypt_ecb(const int algorithm, void * key,const int lenofkey);
 int mcrypt_ecb(int thread, void *plaintext,int len);
 int mdecrypt_ecb(int thread, void *plaintext, int);
 
@@ -26,7 +26,6 @@ int mdecrypt_cfb(int td, void *ciphertext, int len);
 int init_mcrypt_cfb(const int algorithm, void * key,const int lenofkey, void *IV);
 
 
-#ifndef BLOWFISH
 # define BLOWFISH 0
 # define DES 1 
 # define TripleDES 2
@@ -36,4 +35,3 @@ int init_mcrypt_cfb(const int algorithm, void * key,const int lenofkey, void *IV
 # define SAFER128 7
 # define CAST128 8
 # define TEAN 9
-#endif
