@@ -14,8 +14,12 @@
  *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#include "defines.h"
+#ifndef DEFINES_H
+# define DEFINES_H
+# include <defines.h>
+#endif
 #include "bits.h"
+#include <bzero.h>
 
 /* Bit manipulation set/get. I know this are not fast and good but at least
  * they work. Assuming that if a char=1 then his bits are 00000001.
