@@ -1,3 +1,3 @@
 unsigned int getbit(unsigned char bit,unsigned int which);
-char setbit(unsigned char bit,unsigned int which,unsigned int what);
+unsigned char setbit(unsigned char bit,unsigned int which,unsigned int what);
 int strip8bit(char * chain, unsigned int sizeofchain);
