@@ -56,17 +56,6 @@
 #define TRUE 1
 #define FALSE 0
 
-#define BLOWFISH 0
-#define DES 1 
-#define TripleDES 2
-#define ThreeWAY 3
-#define GOST 4
-#define CRYPT 5
-#define SAFER64 6
-#define SAFER128 7
-#define CAST128 8
-#define TEAN 9
-
 #define ENCRYPT 0
 #define DECRYPT 1
 
@@ -74,6 +63,8 @@
 #define ECB 1
 #define CFB 2
 
+#define CRYPT 5
+
 #if SIZEOF_LONG_INT == 4
  typedef unsigned long word32;
 #elif SIZEOF_INT == 4
