@@ -1,8 +1,8 @@
 char * get_password(int numofchar,int bitmode,int mode,int type);
 int show_mode(int ende, int stype,int smode,char *output,int bitmode);
-int BreakToThree(void * key, unsigned int keylen, void * keyword1, void * keyword2, void * keyword3);
+
 char *my_getpass();
-word32 byteswap(word32 x);
+
 int check_file_head(char * filename, unsigned int *algorithm, unsigned int *mode, unsigned int *bit7_flag,unsigned int stream_flag);
 int write_file_head(int filedes, unsigned int algorithm, unsigned int mode, unsigned int bit7_flag);
 
