@@ -1,6 +1,6 @@
 #ifndef DEFINES_H
  #define DEFINES_H
- #include "defines.h"
+ #include <defines.h>
 #endif
 
 void time_out(int signo);
