@@ -410,27 +410,41 @@ for(i=0;i<file_count;i++) {
              }
            }
 
-		      switch (type) {
-		        case DES:
-		        case TripleDES:
-			case ThreeWAY:
-			case BLOWFISH:
-			case GOST:
-			case SAFER64:
-			case SAFER128:
-                        case CAST128:
-                        case TEAN:
-			 x=decrypt_general(type,dinfile,outfile,keyword);
-			 break;
-			case CRYPT:
-			 x=encrypt_ucrypt(dinfile,outfile,keyword);
-			 break;			 
-                        default:
-                         fprintf(stderr,_("Algorithm unsupported in this version\n"));
-                         x=1;
-                         break;
-                        }
+                      switch(mode) {
+                        case CBC:
+                        case ECB:
+                        case CFB:
+
+  			      switch (type) {
+			        case DES:
+			        case TripleDES:
+				case ThreeWAY:
+				case BLOWFISH:
+				case GOST:
+				case SAFER64:
+				case SAFER128:
+        	                case CAST128:
+	                        case TEAN:
+				 x=decrypt_general(type,dinfile,outfile,keyword);
+				 break;
+				case CRYPT:
+				 x=encrypt_ucrypt(dinfile,outfile,keyword);
+				 break;			 
+                        	default:
+	                         fprintf(stderr,_("Algorithm unsupported in this version\n"));
+        	                 x=1;
+                	         break;
+	                      }
                         
+
+                          break;
+                        default:
+                          x=1;
+                          fprintf(stderr,_("Mode unsupported in this version\n"));
+                          break;
+                      }
+                      
+
 			if (x==0) {
         	            if (stream_flag==FALSE) {
         	              if (quiet==FALSE) fprintf (stderr,_("File %s was decrypted.\n"),dinfile);
@@ -460,6 +474,11 @@ for(i=0;i<file_count;i++) {
 
 /* Encrypt */
 	if (ein==TRUE) {
+                  switch(mode) {
+                    case CBC:
+                    case ECB:
+                    case CFB:
+
 		      switch (type) {
 		        case DES:
 		        case TripleDES:
@@ -478,9 +497,17 @@ for(i=0;i<file_count;i++) {
 	        	 break;
 	        	 
 	        	default:
+                         fprintf(stderr,_("Algorithm unsupported in this version\n"));
 	        	 x=1;
 	        	 break;
                         }
+                   break;
+
+                   default:
+                    x=1;
+                    fprintf(stderr,_("Mode unsupported in this version\n"));
+                    break;
+                  }
 
 	        	if (x==0) {
                           if (stream_flag==FALSE) {
@@ -727,16 +754,9 @@ if (mode==CBC || mode==CFB) {
  if (quiet==FALSE) show_mode(ENCRYPT,algorithm,mode,tofile,bit7_flag);
 
 
-  if (algorithm != BLOWFISH) {
-    len=keysize; /* Only Blowfish has non-fixed key */
-  }
-  else { /* blowfish */
-    if (len<2) {
-    fprintf(stderr, _("Too small key for this algorithm. Aborting.\n"));
-    return 1;
-    }
-  }
-
+//  if (algorithm != BLOWFISH) {
+//    len=keysize; /* Only Blowfish has non-fixed key */
+//  }
 
 
   switch (mode) {
@@ -1034,15 +1054,6 @@ else {
 if (quiet==FALSE) show_mode(DECRYPT,algorithm,mode,tofile,bit7_flag);
 
 
- if (algorithm!=BLOWFISH) {
-  len=keysize;
- }
- else { /* blowfish */
-   if (len<2) {
-    fprintf(stderr, _("Too small key for this algorithm. Aborting.\n"));
-    return 1;
-    }
- }
  
   switch (mode) {
     case CBC:
@@ -1258,6 +1269,9 @@ switch (mode) {
     case ECB:
       end_mcrypt_ecb(td);
       break;
+    case CFB:
+      end_mcrypt_cfb(td);
+      break;      
   }
 
   free(ciphertext_old);
